"""
File: queue2.py

Circular-array implementation
"""

from arrays import Array

class CircularArrayQueue(object):
    """ Circular Array-based queue implementation."""

    DEFAULT_CAPACITY = 10   # Class variable applies to all queues
    
    def __init__(self):
        self._items = Array(CircularArrayQueue.DEFAULT_CAPACITY)
        self._rear = -1
        self._front = 0
        self._size = 0

    def enqueue(self, newItem):
        """Adds newItem to the rear of queue."""
        # Resize array if necessary to double its physical size
        if self._size == len(self._items):
            temp = Array(2 * self._size)
            for i in xrange(self._size):
                temp[i] = self._items[self._front]
                self._front = (self._front + 1 ) % len(self._items)
            self._items = temp
            self._front = 0
            self._rear = self._size - 1
            
        # newItem goes at logical end of array
        self._rear = (self._rear + 1 ) % len(self._items)
        self._size += 1
        self._items[self._rear] = newItem

    def dequeue(self):
        """Removes and returns the item at front of the queue.
        Precondition: the queue is not empty."""

        # ADD CODE HERE - BE SURE TO CHECK THE PRECONDITION AND
        # RESIZE TO A SMALLER ARRAY IF WE DROP BELOW A QUARTER
        # ITEMS OF THE PHYSICAL SIZE.  HOWEVER, NEVER DROP BELOW THE
        # DEFAULT_CAPACITY


        
    def peek(self):
        """Returns the item at front of the queue.
        Precondition: the queue is not empty."""
        if self._size == 0:
            raise ValueError, "Queue is empty, so no front item to peek at!"
        
        return self._items[self._front]

    def __len__(self):
        """Returns the number of items in the queue."""
        return self._size

    def isEmpty(self):
        return len(self) == 0

    def __str__(self):
        """Items strung from front to rear."""
        result = ""
        probe = self._front
        for i in xrange(len(self)):
            result += str(self._items[probe]) + " "
            probe = (probe + 1) % len(self._items)
        return result


